@@ -1718,6 +1718,71 @@ ChangeSelectedThermals (int types, int therm_style)
   return (change);
 }
 
+
+/*!
+ * \brief Changes the thermals on all selected and visible pins and/or
+ * vias.
+ *
+ * \return True if anything has changed.
+ */
+bool
+ChangeSelectedViaLayers (int from, int to)
+{
+  bool change = false;
+  int new_from;
+  int new_to;
+  int i;
+
+  VIA_LOOP (PCB->Data);
+    {
+      if (TEST_FLAG (LOCKFLAG, via))
+        continue;
+      if (TEST_FLAG (SELECTEDFLAG, via))
+        {
+
+          new_from = (from != -1)?from:via->BuriedFrom;
+          new_to = (to != -1)?to:via->BuriedTo;
+
+	  if (new_from == new_to)
+	    continue;
+
+          /* special case - changing TH via "from" layer sets "to" layer to bottom layer */
+          if (!VIA_IS_BURIED (via)
+               && (to == -1))
+             new_to = GetMaxBottomLayer ();
+
+          for (i=0; i < max_copper_layer; i++)
+            {
+              /* AddObjectToClearPolyUndoList (VIA_TYPE, &(PCB->Data->Layer[i]), via, via, false); not needed? */
+              RestoreToPolygon (PCB->Data, VIA_TYPE, &(PCB->Data->Layer[i]), via);
+            }
+
+	    AddObjectToSetViaLayersUndoList (via, via, via);
+            via->BuriedFrom = new_from;
+            via->BuriedTo = new_to;
+
+            if (VIA_IS_BURIED (via))
+	      {
+	        SanitizeBuriedVia (via);
+                for (i=0; i < max_copper_layer; i++)
+                  {
+                    /* AddObjectToClearPolyUndoList (VIA_TYPE, &(PCB->Data->Layer[i]), via, via, true);  not needed? */
+                    ClearFromPolygon (PCB->Data, VIA_TYPE, &(PCB->Data->Layer[i]), via);
+                  }
+                DrawVia (via);
+	      }
+            change = true;
+	}
+    }
+  END_LOOP;
+  if (change)
+    {
+      Draw ();
+      IncrementUndoSerialNumber ();
+    }
+  return (change);
+}
+
 /*!
  * \brief Changes the size of all selected and visible object types.
  *
@@ -2096,6 +2161,68 @@ ChangeObjectThermal (int Type, void *Ptr1, void *Ptr2, void *Ptr3,
   return (change);
 }
 
+/*!
+ * \brief Changes the thermal of the passed object.
+ *
+ * \return True if anything is changed.
+ *
+ */
+bool
+ChangeObjectViaLayers (void *Ptr1, void *Ptr2, void *Ptr3,
+		     int from, int to)
+{
+  bool change = false;
+  PinType *via = (PinType *) Ptr1;
+  int new_from = (from != -1)?from:via->BuriedFrom;
+  int new_to = (to != -1)?to:via->BuriedTo;
+  int i;
+
+  if (TEST_FLAG (LOCKFLAG, via))
+    return (NULL);
+
+  if ((new_from == new_to)
+      && new_from != 0)
+    return false;
+
+  /* special case - changing TH via "from" layer sets "to" layer to bottom layer */
+  if (!VIA_IS_BURIED (via)
+       && (to == -1))
+     new_to = GetMaxBottomLayer ();
+
+  for (i=0; i < max_copper_layer; i++)
+    {
+      /* AddObjectToClearPolyUndoList (VIA_TYPE, &(PCB->Data->Layer[i]), via, via, false);  not needed? */
+      RestoreToPolygon (PCB->Data, VIA_TYPE, &(PCB->Data->Layer[i]), via);
+    }
+
+  if (from != -1 || to != -1)
+    {
+      AddObjectToSetViaLayersUndoList (via, via, via);
+      change = true;
+    }
+
+  via->BuriedFrom = new_from;
+  via->BuriedTo = new_to;
+
+  if (VIA_IS_BURIED (via))
+    {
+      SanitizeBuriedVia (via);
+      for (i=0; i < max_copper_layer; i++)
+        {
+          /* AddObjectToClearPolyUndoList (VIA_TYPE, &(PCB->Data->Layer[i]), via, via, true);  not needed? */
+          ClearFromPolygon (PCB->Data, VIA_TYPE, &(PCB->Data->Layer[i]), via);
+        }
+      DrawVia (via);
+    }
+
+  if (change)
+    {
+      Draw ();
+      IncrementUndoSerialNumber ();
+    }
+  return (change);
+}
+
 /*!
  * \brief Changes the 2nd size of the passed object.
  *
